@@ -24,13 +24,14 @@ import requests
 import six
 from six.moves import urllib
 import subprocess
-import time
+import tenacity
 
 import docker
 try:
     from docker import APIClient as Client
 except ImportError:
     from docker import Client
+from oslo_concurrency import processutils
 from tripleo_common.image.base import BaseImageManager
 from tripleo_common.image.exception import ImageUploaderException
 
@@ -173,20 +174,23 @@ class DockerImageUploader(ImageUploader):
             LOG.info('Skipping upload for image %s' % image_name)
             return []
 
-        DockerImageUploader._pull_retry(dockerc, repo, tag=tag)
+        DockerImageUploader._pull(dockerc, repo, tag=tag)
 
         response = dockerc.tag(image=full_image, repository=new_repo,
                                tag=tag, force=True)
         LOG.debug(response)
 
-        response = [line for line in dockerc.push(new_repo,
-                    tag=tag, stream=True)]
-        LOG.debug(response)
+        DockerImageUploader._push(dockerc, new_repo, tag=tag)
 
         LOG.info('Completed upload for image %s' % image_name)
         return full_image, full_new_repo
 
     @staticmethod
+    @tenacity.retry(  # Retry up to 5 times with jittered exponential backoff
+        reraise=True,
+        wait=tenacity.wait_random_exponential(multiplier=1, max=10),
+        stop=tenacity.stop_after_attempt(5)
+    )
     def _pull(dockerc, image, tag=None):
         LOG.debug('Pulling %s' % image)
 
@@ -194,22 +198,22 @@ class DockerImageUploader(ImageUploader):
             status = json.loads(line)
             if 'error' in status:
                 LOG.warning('docker pull failed: %s' % status['error'])
-                return 1
-            LOG.debug(status.get('status'))
-        return 0
+                raise ImageUploaderException('Could not pull image %s' % image)
 
     @staticmethod
-    def _pull_retry(dockerc, image, tag=None):
-        retval = -1
-        count = 0
-        while retval != 0:
-            if count >= 5:
-                raise ImageUploaderException('Could not pull image %s' % image)
-            count += 1
-            retval = DockerImageUploader._pull(dockerc, image, tag)
-            if retval != 0:
-                time.sleep(3)
-                LOG.warning('retrying pulling image: %s' % image)
+    @tenacity.retry(  # Retry up to 5 times with jittered exponential backoff
+        reraise=True,
+        wait=tenacity.wait_random_exponential(multiplier=1, max=10),
+        stop=tenacity.stop_after_attempt(5)
+    )
+    def _push(dockerc, image, tag=None):
+        LOG.debug('Pushing %s' % image)
+
+        for line in dockerc.push(image, tag=tag, stream=True):
+            status = json.loads(line)
+            if 'error' in status:
+                LOG.warning('docker push failed: %s' % status['error'])
+                raise ImageUploaderException('Could not push image %s' % image)
 
     @staticmethod
     def _images_match(image1, image2, insecure_registries):
@@ -263,7 +267,8 @@ class DockerImageUploader(ImageUploader):
         return urllib.parse.urlparse(image)
 
     @staticmethod
-    def _discover_tag_from_inspect(i, image, tag_from_label=None):
+    def _discover_tag_from_inspect(i, image, tag_from_label=None,
+                                   fallback_tag=None):
         labels = i.get('Labels', {})
 
         label_keys = ', '.join(labels.keys())
@@ -273,12 +278,29 @@ class DockerImageUploader(ImageUploader):
                 'No label specified. Available labels: %s' % label_keys
             )
 
-        tag_label = labels.get(tag_from_label)
-        if tag_label is None:
-            raise ImageUploaderException(
-                'Image %s has no label %s. Available labels: %s' %
-                (image, tag_from_label, label_keys)
-            )
+        if "{" in tag_from_label:
+            try:
+                tag_label = tag_from_label.format(**labels)
+            except ValueError as e:
+                raise ImageUploaderException(e)
+            except KeyError as e:
+                if fallback_tag:
+                    tag_label = fallback_tag
+                else:
+                    raise ImageUploaderException(
+                        'Image %s %s. Available labels: %s' %
+                        (image, e, label_keys)
+                    )
+        else:
+            tag_label = labels.get(tag_from_label)
+            if tag_label is None:
+                if fallback_tag:
+                    tag_label = fallback_tag
+                else:
+                    raise ImageUploaderException(
+                        'Image %s has no label %s. Available labels: %s' %
+                        (image, tag_from_label, label_keys)
+                    )
 
         # confirm the tag exists by checking for an entry in RepoTags
         repo_tags = i.get('RepoTags', [])
@@ -308,11 +330,13 @@ class DockerImageUploader(ImageUploader):
             versioned_images[image] = versioned_image
         return versioned_images
 
-    def discover_image_tag(self, image, tag_from_label=None):
+    def discover_image_tag(self, image, tag_from_label=None,
+                           fallback_tag=None):
         image_url = self._image_to_url(image)
         insecure = self.is_insecure_registry(image_url.netloc)
         i = self._inspect(image_url.geturl(), insecure)
-        return self._discover_tag_from_inspect(i, image, tag_from_label)
+        return self._discover_tag_from_inspect(i, image, tag_from_label,
+                                               fallback_tag)
 
     def cleanup(self, local_images):
         dockerc = Client(base_url='unix://var/run/docker.sock', version='auto')
@@ -349,7 +373,9 @@ class DockerImageUploader(ImageUploader):
         result = self.upload_image(*first)
         local_images.extend(result)
 
-        p = multiprocessing.Pool(4)
+        # workers will be half the CPU count, to a minimum of 2
+        workers = max(2, processutils.get_worker_count() // 2)
+        p = multiprocessing.Pool(workers)
 
         for result in p.map(docker_upload, self.upload_tasks):
             local_images.extend(result)
@@ -387,5 +413,12 @@ def discover_tag_from_inspect(args):
     image_url = DockerImageUploader._image_to_url(image)
     insecure = image_url.netloc in insecure_registries
     i = DockerImageUploader._inspect(image_url.geturl(), insecure)
+    if ':' in image_url.path:
+        # break out the tag from the url to be the fallback tag
+        path = image.rpartition(':')
+        fallback_tag = path[2]
+        image = path[0]
+    else:
+        fallback_tag = None
     return image, DockerImageUploader._discover_tag_from_inspect(
-        i, image, tag_from_label)
+        i, image, tag_from_label, fallback_tag)
